@@ -122,7 +122,9 @@ XEXT_GENERATE_FIND_DISPLAY(XInput_find_display, xinput_info,
     {XI_Present, XI_Add_XSetDeviceValuators_Major,
      XI_Add_XSetDeviceValuators_Minor},
     {XI_Present, XI_Add_XChangeDeviceControl_Major,
-     XI_Add_XChangeDeviceControl_Minor}
+     XI_Add_XChangeDeviceControl_Minor},
+    {XI_Present, XI_Add_DevicePresenceNotify_Major,
+     XI_Add_DevicePresenceNotify_Minor}
     };
 
 /***********************************************************************
@@ -189,12 +191,12 @@ _xidevicebusy(dpy, error)
  */
 
 int
-_XiCheckExtInit(dpy, version_index)
+_XiCheckExtInit(dpy, version_index, info)
     register Display *dpy;
     register int version_index;
+    XExtDisplayInfo *info;
 {
     XExtensionVersion *ext;
-    XExtDisplayInfo *info = XInput_find_display(dpy);
 
     XInputCheckExtension(dpy, info, -1);
 
@@ -205,7 +207,7 @@ _XiCheckExtInit(dpy, version_index)
 	    return (-1);
 	}
 	((XInputData *) info->data)->vers =
-	    _XiGetExtensionVersion(dpy, "XInputExtension");
+	    _XiGetExtensionVersion(dpy, "XInputExtension", info);
     }
 
     if (versions[version_index].major_version > Dont_Check) {
@@ -251,6 +253,14 @@ Ones(mask)
     return (((y + (y >> 3)) & 030707070707) % 077);
 }
 
+static int
+_XiGetDevicePresenceNotifyEvent(Display * dpy)
+{
+    XExtDisplayInfo *info = XInput_find_display(dpy);
+
+    return info->codes->first_event + XI_DevicePresenceNotify;
+}
+
 /***********************************************************************
  *
  * Handle Input extension events.
@@ -665,6 +675,22 @@ XInputWireToEvent(dpy, re, event)
 	return (ENQUEUE_EVENT);
     }
 	break;
+
+    case XI_DevicePresenceNotify:
+    {
+	XDevicePresenceNotifyEvent *ev = (XDevicePresenceNotifyEvent *) re;
+	devicePresenceNotify *ev2 = (devicePresenceNotify *) event;
+
+	*ev = *(XDevicePresenceNotifyEvent *) save;
+	ev->window = 0;
+	ev->time = ev2->time;
+        ev->devchange = ev2->devchange;
+        ev->deviceid = ev2->deviceid;
+        ev->control = ev2->control;
+	return (ENQUEUE_EVENT);
+    }
+	break;
+
     default:
 	printf("XInputWireToEvent: UNKNOWN WIRE EVENT! type=%d\n", type);
 	break;
